Package: dtwclust
Type: Package
Title: Time Series Clustering Along with Optimizations for the Dynamic
        Time Warping Distance
Description: Time series clustering along with optimized techniques related
    to the Dynamic Time Warping distance and its corresponding lower bounds.
    Implementations of partitional, hierarchical, fuzzy, k-Shape and TADPole
    clustering are available. Functionality can be easily extended with
    custom distance measures and centroid definitions. Implementations of
    DTW barycenter averaging, a distance based on global alignment kernels,
    and the soft-DTW distance and centroid routines are also provided. 
    All included distance functions have custom loops optimized for the 
    calculation of cross-distance matrices, including parallelization support.
    Several cluster validity indices are included.
Version: 5.5.7
Depends: R (>= 3.3.0), methods, proxy (>= 0.4-16), dtw
Imports: parallel, stats, utils, clue, cluster, dplyr, flexclust,
        foreach, ggplot2, ggrepel, Matrix, RSpectra, Rcpp, RcppParallel
        (>= 4.4.0), reshape2, shiny, shinyjs
LinkingTo: Rcpp, RcppArmadillo, RcppParallel, RcppThread
Suggests: doParallel, knitr, rmarkdown, testthat
Date: 2022-01-13
Author: Alexis Sarda-Espinosa
Maintainer: Alexis Sarda <alexis.sarda@gmail.com>
BugReports: https://github.com/asardaes/dtwclust/issues
License: GPL-3
URL: https://github.com/asardaes/dtwclust
NeedsCompilation: yes
SystemRequirements: C++11, GNU make
Encoding: UTF-8
LazyData: TRUE
RoxygenNote: 7.1.2
VignetteBuilder: knitr
Collate: 'CENTROIDS-dba.R' 'CENTROIDS-pam.R' 'CENTROIDS-sdtw-cent.R'
        'CENTROIDS-shape-extraction.R' 'CLUSTERING-all-cent2.R'
        'CLUSTERING-compare-clusterings.R'
        'CLUSTERING-cvi-evaluators.R' 'CLUSTERING-ddist2.R'
        'CLUSTERING-partitional-fuzzy.R'
        'CLUSTERING-repeat-clustering.R' 'CLUSTERING-tadpole.R'
        'CLUSTERING-tsclust-controls.R' 'CLUSTERING-tsclust.R'
        'DISTANCES-dtw-basic.R' 'DISTANCES-dtw-lb.R' 'DISTANCES-dtw2.R'
        'DISTANCES-gak.R' 'DISTANCES-lb-improved.R'
        'DISTANCES-lb-keogh.R' 'DISTANCES-sbd.R' 'DISTANCES-sdtw.R'
        'GENERICS-cvi.R' 'S4-Distmat.R' 'S4-PairTracker.R'
        'S4-SparseDistmat.R' 'S4-tsclustFamily.R'
        'S4-TSClusters-classes.R' 'S4-TSClusters-methods.R'
        'SHINY-interactive-clustering.R' 'SHINY-ssdtwclust.R'
        'SHINY-utils.R' 'UTILS-as-methods.R' 'UTILS-compute-envelope.R'
        'UTILS-data.R' 'UTILS-expressions.R' 'UTILS-globals-internal.R'
        'UTILS-nccc.R' 'UTILS-reinterpolate.R' 'UTILS-rng.R'
        'UTILS-tslist.R' 'UTILS-utils.R' 'UTILS-zscore.R' 'pkg.R'
Packaged: 2022-01-12 23:43:40 UTC; oso
Repository: CRAN
Date/Publication: 2022-01-13 00:52:41 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2022-01-13 05:08:11 UTC; windows
Archs: x64
