# Generated by roxygen2: do not edit by hand

S3method(as.cl_membership,TSClusters)
S3method(as.data.frame,crossdist)
S3method(as.data.frame,pairdist)
S3method(as.matrix,crossdist)
S3method(as.matrix,pairdist)
S3method(cl_class_ids,TSClusters)
S3method(cl_membership,TSClusters)
S3method(is.cl_dendrogram,TSClusters)
S3method(is.cl_hard_partition,TSClusters)
S3method(is.cl_hierarchy,TSClusters)
S3method(is.cl_partition,TSClusters)
S3method(n_of_classes,TSClusters)
S3method(n_of_objects,TSClusters)
S3method(plot,TSClusters)
S3method(predict,TSClusters)
S3method(update,TSClusters)
export(DBA)
export(GAK)
export(NCCc)
export(SBD)
export(TADPole)
export(as.matrix)
export(compare_clusterings)
export(compare_clusterings_configs)
export(compute_envelope)
export(cvi)
export(cvi_evaluators)
export(dba)
export(dtw2)
export(dtw_basic)
export(dtw_lb)
export(fuzzy_control)
export(gak)
export(hierarchical_control)
export(interactive_clustering)
export(lb_improved)
export(lb_keogh)
export(pam_cent)
export(partitional_control)
export(pdc_configs)
export(reinterpolate)
export(repeat_clustering)
export(sbd)
export(sdtw)
export(sdtw_cent)
export(shape_extraction)
export(ssdtwclust)
export(tadpole)
export(tadpole_control)
export(tsclust)
export(tsclust_args)
export(tslist)
export(zscore)
exportClasses(FuzzyTSClusters)
exportClasses(HierarchicalTSClusters)
exportClasses(PartitionalTSClusters)
exportClasses(TSClusters)
exportClasses(tsclustFamily)
exportMethods(cvi)
exportMethods(plot)
exportMethods(predict)
exportMethods(show)
exportMethods(update)
import(foreach)
importClassesFrom(Matrix,sparseMatrix)
importFrom(Matrix,forceSymmetric)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(RSpectra,eigs_sym)
importFrom(Rcpp,evalCpp)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(RcppParallel,defaultNumThreads)
importFrom(clue,as.cl_class_ids)
importFrom(clue,as.cl_membership)
importFrom(clue,cl_class_ids)
importFrom(clue,cl_membership)
importFrom(clue,is.cl_dendrogram)
importFrom(clue,is.cl_hard_partition)
importFrom(clue,is.cl_hierarchy)
importFrom(clue,is.cl_partition)
importFrom(clue,n_of_classes)
importFrom(clue,n_of_objects)
importFrom(cluster,silhouette)
importFrom(dplyr,anti_join)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,inner_join)
importFrom(dplyr,sample_n)
importFrom(dtw,dtw)
importFrom(dtw,symmetric1)
importFrom(dtw,symmetric2)
importFrom(flexclust,comPart)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_bw)
importFrom(ggrepel,geom_label_repel)
importFrom(graphics,plot)
importFrom(methods,S3Part)
importFrom(methods,callNextMethod)
importFrom(methods,initialize)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,setOldClass)
importFrom(methods,setRefClass)
importFrom(methods,show)
importFrom(methods,signature)
importFrom(methods,slot)
importFrom(parallel,nextRNGStream)
importFrom(parallel,splitIndices)
importFrom(proxy,dist)
importFrom(proxy,pr_DB)
importFrom(reshape2,melt)
importFrom(shiny,runApp)
importFrom(shiny,shinyApp)
importFrom(shinyjs,useShinyjs)
importFrom(stats,approx)
importFrom(stats,as.dist)
importFrom(stats,as.hclust)
importFrom(stats,convolve)
importFrom(stats,cutree)
importFrom(stats,fft)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,nextn)
importFrom(stats,optim)
importFrom(stats,predict)
importFrom(stats,runif)
importFrom(stats,update)
importFrom(utils,head)
importFrom(utils,packageVersion)
useDynLib(dtwclust, .registration = TRUE)
